# -*- encoding: utf-8 -*-
import struct
import logging

import SEAScope.cmds.get_extraction
import SEAScope.lib.utils
import SEAScope.upload
try:
    import cPickle as pickle
except ImportError:
    import pickle

logger = logging.getLogger(__name__)


def get_dists(mask):
    """"""
    import pyproj
    geod = pyproj.Geod(ellps='WGS84')
    gcps = mask['meta']['gcps']

    dists = [0.0]
    last_lon = gcps[0]['lon']
    last_lat = gcps[0]['lat']
    logger.debug('Number of steps: {}'.format(len(gcps)))

    for n in range(1, len(gcps)):
        lon = gcps[n]['lon']
        lat = gcps[n]['lat']
        _, _, d = geod.inv(last_lon, last_lat, lon, lat)
        dists.append(dists[-1] + .001 * d)
        last_lon = lon
        last_lat = lat

    return dists


def arrays2rgb(channels):
    """"""
    ubyte_data = []
    for n in range(0,3):
        d_min = numpy.min(channels[n])
        d_max = numpy.max(channels[n])
        extent = d_min - d_max
        if d_min == d_max:
            extent = 1.0
        d_ubyte = 255 * (channels[n] - d_min) / extent
        ubyte_data.append(d_ubyte)
    rgb = numpy.stack(ubyte_data, axis=2).astype(numpy.ubyte)
    return rgb


def read_response(s, msg_size):
    """"""
    chunk_size = msg_size
    already_read = 0
    result = b''
    while already_read < msg_size:
        buf = s.recv(chunk_size)
        already_read += len(buf)
        result += buf
        chunk_size = msg_size - already_read

    return result


def get_extracted_data(host='localhost', port=11155):
    """"""
    logger.info('Retrieving extracted data...')
    with SEAScope.upload.connect(host, port) as s:
        serializer = SEAScope.cmds.get_extraction.serialize
        builder = None
        builder, serialized = serializer(builder)
        builder.Finish(serialized)
        buf = builder.Output()
        s.sendall(struct.pack('>Q', len(buf))+buf)

        buf = s.recv(8)
        l = struct.unpack('<Q', buf)
        logger.debug('Read {} bytes'.format(l[0]))
        buf = read_response(s, l[0])
        logger.debug('{} bytes read'.format(len(buf)))
        deserializer = SEAScope.cmds.get_extraction.deserialize
        raw_result = deserializer(buf)

    clean_data = SEAScope.lib.utils.raw2data(raw_result)
    del raw_result
    return clean_data


def save_pyo(obj, output_path):
    """"""
    with open(output_path, 'wb') as f:
        pickle.dump(obj, f, protocol=1)


def load_pyo(input_path):
    """"""
    obj = None
    with open(input_path, 'rb') as f:
        obj = pickle.load(f)
    return obj
