
def plot_2d_data(source_name, field_name, field_data, output_path):
    """"""
    import matplotlib
    import matplotlib.pyplot
    slice_size = 100
    slices = range(0, len(source_name), slice_size)
    wrapped_srcname = [source_name[i:i+slice_size] for i in slices]

    fig = matplotlib.pyplot.figure(1, figsize=(10, 10), dpi=300)
    ax = fig.add_axes([0, 0, 1, 1], visible=True)
    ax.set_axis_off()
    matplotlib.pyplot.figtext(.5, 1.0, '\n'.join(wrapped_srcname), fontsize=10,
                              ha='center', figure=fig)
    matplotlib.pyplot.figtext(.5, 0.95, field_name, fontsize=16, ha='center',
                              figure=fig)
    _cmap = matplotlib.pyplot.get_cmap('jet')
    ax.imshow(field_data, cmap=_cmap, interpolation='nearest')

    fig.savefig(output_path, bbox_inches='tight')
    matplotlib.pyplot.close()


def plot_transect(dists, transects, output_path):
    """"""
    import matplotlib
    import matplotlib.pyplot
    cmap = matplotlib.pyplot.get_cmap('hsv', 1 + len(transects.keys()))

    fig = matplotlib.pyplot.figure(1, figsize=(10, 10), dpi=300)
    ax = fig.add_axes([0, 0, 1, 1], visible=True)
    ax.set_xlabel('Distance (km)')

    k = 0
    for uri in transects:
        slice_size = 120
        slices = range(0, len(uri), slice_size)
        wrapped_uri = [uri[i:i+slice_size] for i in slices]

        ax.plot(dists, transects[uri], c=cmap(k), label='\n'.join(wrapped_uri))
        k = k + 1

    legend = matplotlib.pyplot.legend(bbox_to_anchor=(0., 1.02, 1., .102),
                                      loc=3, ncol=1, mode="expand",
                                      borderaxespad=0.)
    matplotlib.pyplot.setp(legend.get_title(),fontsize='xx-small')

    fig.savefig(output_path, bbox_inches='tight')
    matplotlib.pyplot.close()
