# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-09
"""

import flatbuffers
import SEAScope.API.GranuleMetadata
import SEAScope.API.DataModel
import SEAScope.types.idf_descriptor

data_models = {'LAT_LON': SEAScope.API.DataModel.DataModel.LAT_LON,
               'Y_X': SEAScope.API.DataModel.DataModel.Y_X,
               'NJ_NI': SEAScope.API.DataModel.DataModel.NJ_NI,
               'ROW_CELL': SEAScope.API.DataModel.DataModel.ROW_CELL,
               'TIME_STATION': SEAScope.API.DataModel.DataModel.TIME_STATION,
               'TIME': SEAScope.API.DataModel.DataModel.TIME}


def serialize(builder, obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.GranuleMetadata

    src_id = int(obj['sourceId'])
    col_id = int(obj['collectionId'])
    gra_id = int(obj['granuleId'])
    data_id = builder.CreateString(obj['dataId'])
    data_model = data_models.get(obj['dataModel'], None)
    start = obj['start']
    stop = obj['stop']

    uris = []
    l = len(obj['uris'])
    for _uri in obj['uris']:
        _, idf_descriptor = SEAScope.types.idf_descriptor.serialize(builder,
                                                                    _uri)
        uris.append(idf_descriptor)
    cls.GranuleMetadataStartUrisVector(builder, l)
    for i in uris[::-1]:
        builder.PrependUOffsetTRelative(i)
    _uris = builder.EndVector(l)

    # optional attributes
    title = builder.CreateString(obj.get('title', ''))
    institution = builder.CreateString(obj.get('institution', ''))
    comment = builder.CreateString(obj.get('comment', ''))
    file_id = builder.CreateString(obj.get('file_id', ''))
    product_version = builder.CreateString(obj.get('product_version', ''))
    lat_min = obj.get('lat_min', 0.0)
    lat_max = obj.get('lat_max', 0.0)
    lon_min = obj.get('lon_min', 0.0)
    lon_max = obj.get('lon_max', 0.0)
    creator_email = builder.CreateString(obj.get('creator_email', ''))
    station_id = builder.CreateString(obj.get('station_id', ''))
    platform = builder.CreateString(obj.get('platform', ''))
    sensor = builder.CreateString(obj.get('sensor', ''))

    cls.GranuleMetadataStart(builder)
    cls.GranuleMetadataAddSourceId(builder, src_id)
    cls.GranuleMetadataAddCollectionId(builder, col_id)
    cls.GranuleMetadataAddGranuleId(builder, gra_id)
    cls.GranuleMetadataAddDataId(builder, data_id)
    cls.GranuleMetadataAddDataModel(builder, data_model)
    cls.GranuleMetadataAddStart(builder, start)
    cls.GranuleMetadataAddStop(builder, stop)
    cls.GranuleMetadataAddUris(builder, _uris)
    cls.GranuleMetadataAddHasTitle(builder, 'title' in obj)
    cls.GranuleMetadataAddHasInstitution(builder, 'institution' in obj)
    cls.GranuleMetadataAddHasComment(builder, 'comment' in obj)
    cls.GranuleMetadataAddHasFileId(builder, 'file_id' in obj)
    cls.GranuleMetadataAddHasProductVersion(builder, 'product_version' in obj)
    cls.GranuleMetadataAddHasLatMin(builder, 'lat_min' in obj)
    cls.GranuleMetadataAddHasLatMax(builder, 'lat_max' in obj)
    cls.GranuleMetadataAddHasLonMin(builder, 'lon_min' in obj)
    cls.GranuleMetadataAddHasLonMax(builder, 'lon_max' in obj)
    cls.GranuleMetadataAddHasCreatorEmail(builder, 'creator_email' in obj)
    cls.GranuleMetadataAddHasStationId(builder, 'station_id' in obj)
    cls.GranuleMetadataAddHasPlatform(builder, 'platform' in obj)
    cls.GranuleMetadataAddHasSensor(builder, 'sensor' in obj)
    cls.GranuleMetadataAddTitle(builder, title)
    cls.GranuleMetadataAddInstitution(builder, institution)
    cls.GranuleMetadataAddComment(builder, comment)
    cls.GranuleMetadataAddFileId(builder, file_id)
    cls.GranuleMetadataAddProductVersion(builder, product_version)
    cls.GranuleMetadataAddLatMin(builder, lat_min)
    cls.GranuleMetadataAddLatMax(builder, lat_max)
    cls.GranuleMetadataAddLonMin(builder, lon_min)
    cls.GranuleMetadataAddLonMax(builder, lon_max)
    cls.GranuleMetadataAddCreatorEmail(builder, creator_email)
    cls.GranuleMetadataAddStationId(builder, station_id)
    cls.GranuleMetadataAddPlatform(builder, platform)
    cls.GranuleMetadataAddSensor(builder, sensor)
    gm = cls.GranuleMetadataEnd(builder)
    return builder, gm


def deserialize(o):
    """"""
    result = {}
    result['sourceId'] = o.SourceId()
    result['collectionId'] = o.CollectionId()
    result['granuleId'] = o.GranuleId()
    result['dataId'] = o.DataId()
    data_model = o.DataModel()
    for k, v in data_models.items():
        if v == data_model:
            result['dataModel'] = k
            break
    result['start'] = o.Start()
    result['stop'] = o.Stop()
    result['uris'] = []
    uris_count = o.UrisLength()
    for i in range(0, uris_count):
        u = SEAScope.types.idf_descriptor.deserialize(o.Uris(i))
        result['uris'].append(u)

    if o.HasTitle():
        result['title'] = o.Title()
    if o.HasInstitution():
        result['institution'] = o.Institution()
    if o.HasComment():
        result['comment'] = o.Comment()
    if o.HasFileId():
        result['file_id'] = o.FileId()
    if o.HasProductVersion():
        result['product_version'] = o.ProductVersion()
    if o.HasLatMin():
        result['lat_min'] = o.LatMin()
    if o.HasLatMax():
        result['lat_max'] = o.LatMax()
    if o.HasLonMin():
        result['lon_min'] = o.LonMin()
    if o.HasLonMax():
        result['lon_max'] = o.LonMax()
    if o.HasCreatorEmail():
        result['creator_email'] = o.CreatorEmail()
    if o.HasStationId():
        result['station_id'] = o.StationId()
    if o.HasPlatform():
        result['platform'] = o.Platform()
    if o.HasSensor():
        result['sensor'] = o.Sensor()
    return result
