# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-07
"""

import flatbuffers
import SEAScope.API.Variable
import SEAScope.API.RenderingMethod
import SEAScope.types.tag
import SEAScope.types.rendering_cfg


rendering_methods = SEAScope.types.rendering_cfg.rendering_methods


def serialize(builder, var_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    _id = int(var_obj['id'])
    _label = builder.CreateString(var_obj['label'])
    _units = builder.CreateString(var_obj['units'])
    fields_buffer = []
    for field in var_obj['fields']:
        _field = builder.CreateString(field)
        fields_buffer.append(_field)
    l = len(fields_buffer)
    SEAScope.API.Variable.VariableStartFieldsVector(builder, l)
    for t in fields_buffer:
        builder.PrependUOffsetTRelative(t)
    _fields = builder.EndVector(l)

    tags_buffer = []
    for k, v in var_obj['tags'].items():
        _, _tag = SEAScope.types.tag.serialize(builder, k, v)
        tags_buffer.append(_tag)
    l = len(tags_buffer)
    SEAScope.API.Variable.VariableStartTagsVector(builder, l)
    for t in tags_buffer:
        builder.PrependUOffsetTRelative(t)
    _tags = builder.EndVector(l)

    rendering = var_obj['defaultRenderingMethod']
    _default_rendering = rendering_methods.get(rendering, None)

    SEAScope.API.Variable.VariableStart(builder)
    SEAScope.API.Variable.VariableAddId(builder, _id)
    SEAScope.API.Variable.VariableAddLabel(builder, _label)
    SEAScope.API.Variable.VariableAddUnits(builder, _units)
    SEAScope.API.Variable.VariableAddFields(builder, _fields)
    SEAScope.API.Variable.VariableAddTags(builder, _tags)
    SEAScope.API.Variable.VariableAddDefaultRenderingMethod(builder,
                                                            _default_rendering)
    variable = SEAScope.API.Variable.VariableEnd(builder)
    return builder, variable


def deserialize(v):
    """"""
    v_obj = {}
    v_obj['id'] = v.Id()
    v_obj['label'] = v.Label()
    v_obj['units'] = v.Units()
    fields_count = v.FieldsLength()
    v_obj['fields'] = [v.Fields(x) for x in range(0, fields_count)]
    tags_count = v.TagsLength()
    v_obj['tags'] = {}
    for i in range(0, tags_count):
        name, value = SEAScope.types.tag.deserialize(v.Tags(i))
        v_obj['tags'][name] = value

    v_obj['defaultRenderingMethod'] = 'NONE'
    rendering = v.DefaultRenderingMethod()
    for k, v in rendering_methods.items():
        if v == rendering:
            v_obj['defaultRenderingMethod'] = k
            break
    return v_obj
